#include "GLSLModel.h"


void loadShaderFile(GLuint id, const char *argFileName)
{
	// t@C̒gSēǂݍ
	std::ifstream inputFile1(argFileName);
	std::istreambuf_iterator<char> vdataBegin(inputFile1);
	std::istreambuf_iterator<char> vdataEnd;
	std::string fileData1(vdataBegin,vdataEnd);
	const char *pFile = fileData1.c_str();

	glShaderSource(id, 1, &pFile, NULL);
	glCompileShader(id);
}

void printShaderInfoLog(GLuint shader)
{
	GLsizei bufSize;

	/* VF[_̃RpC̃O̒擾 */
	glGetShaderiv(shader, GL_INFO_LOG_LENGTH , &bufSize);

	if(bufSize > 1) {
		GLchar *infoLog = (GLchar *)malloc(bufSize);

		if(infoLog != NULL) {
			GLsizei length;

			/* VF[_̃RpC̃O̓e擾 */
			glGetShaderInfoLog(shader, bufSize, &length, infoLog);
			fprintf(stderr, "InfoLog:\n%s\n\n", infoLog);
			free(infoLog);
		} else {
			fprintf(stderr, "Could not allocate InfoLog buffer.\n");
		}
	}
}

GLSLModel::GLSLModel(void) : fk_Model()
{
	texID = 0;
	vpID = fpID = shID = 0;
	useTex = useSatBle = false;
	
	return;
}

GLSLModel::~GLSLModel()
{
	glDeleteShader(vpID);
	glDeleteShader(fpID);
	glDeleteProgram(shID);
}

void GLSLModel::initShader(int argVP, int argFP)
{
	// Fl::check();

	vpID = glCreateShader(GL_VERTEX_SHADER);
	loadShaderFile(vpID, "shader/model_vp.glsl");
	printShaderInfoLog(vpID);

	fpID = glCreateShader(GL_FRAGMENT_SHADER);
	if(argFP == 0) {
		loadShaderFile(fpID, "shader/model_fp.glsl");
	} else if(argFP == 1) {
		loadShaderFile(fpID, "shader/modelTex_fp.glsl");
		useTex = true;
	} else if(argFP == 2) {
		loadShaderFile(fpID, "shader/shockSph_fp.glsl");
		useSatBle = true;
	}
	printShaderInfoLog(fpID);

	shID = glCreateProgram();
	glAttachShader(shID, vpID);
	glAttachShader(shID, fpID);

	GLint	linked = 0;
	glLinkProgram(shID);
	glGetProgramiv(shID, GL_LINK_STATUS, &linked);
	if(linked == GL_FALSE) {
		std::cerr << "Link Err.\n";
	}

	if(useTex) {
		glUniform1i(glGetUniformLocation(shID, "tex0"), 0);
	}

	return;
}

void GLSLModel::preShader(void)
{
	if(useSatBle) glBlendFunc(GL_ONE, GL_ONE);
	glUseProgram(shID);

	return;
}	

void GLSLModel::postShader(void)
{
	glUseProgram(0);
	if(useSatBle) glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	return;
}

void GLSLModel::connectShader(fk_TexID argID)
{
	texID = argID;
}
